function [theta_model_f, theta_pre_f, L] = train_theta(xlf, feature_info, yf, seq, params, theta_pre_f, theta_model_f)

if seq.frame == 1
    theta_pre_f = cell(size(xlf));
    theta_model_f = cell(size(xlf));
    L = params.mask_window;
end


for k = 1: numel(xlf)
    model_xf = gather(xlf{k});
    
    if (seq.frame == 1)
        theta_pre_f{k} = zeros(size(model_xf));
        theta_model_f{k} = zeros(size(model_xf));
        lambda2 = 0;
    else
        lambda2 = params.temporal_consistency_factor(feature_info.feature_is_deep(k)+1);
    end
    
    % intialize the variables
    f_f = single(zeros(size(model_xf)));
    g_f = f_f;
    h_f = f_f;
    mu  = params.init_penalty_factor;
    mu_scale_step = params.penalty_scale_step;
    
    
    % pre-compute the variables
    T = feature_info.data_sz(k)^2;
    S_xx = sum(conj(model_xf) .* model_xf, 3);
    Stheta_pre_f = sum(conj(model_xf) .* theta_pre_f{k}, 3);
    Sfx_pre_f = bsxfun(@times, model_xf, Stheta_pre_f);
    
    % solve via ADMM algorithm
    iter = 1;
    while (iter <= params.max_iterations)
        
        % subproblem f
        B = S_xx + T * (mu + lambda2);
        Sgx_f = sum(conj(model_xf) .* g_f, 3);
        Shx_f = sum(conj(model_xf) .* h_f, 3);
        
        f_f = ((1/(T*(mu + lambda2)) * bsxfun(@times,  yf{k}, model_xf)) - ((1/(mu + lambda2)) * h_f) +(mu/(mu + lambda2)) * g_f) + (lambda2/(mu + lambda2)) * theta_pre_f{k} - ...
            bsxfun(@rdivide,(1/(T*(mu + lambda2)) * bsxfun(@times, model_xf, (S_xx .*  yf{k})) + (lambda2/(mu + lambda2)) * Sfx_pre_f - ...
            (1/(mu + lambda2))* (bsxfun(@times, model_xf, Shx_f)) +(mu/(mu + lambda2))* (bsxfun(@times, model_xf, Sgx_f))), B);
        
        %   subproblem g
        X = real(ifft2(mu * f_f+ h_f));
        if (seq.frame == 1)
            T1 = zeros(size(X));
            for i = 1:size(X,3)
                T1(:,:,i) = params.mask_window{k} .* X(:,:,i);
            end
        else
            T1=X;
            
            L{k} = max(0,1-mu./(numel(X)*sqrt(sum(T1.^2,3))));
            
            [~,b] = sort(L{k}(:),'descend');
            
            L{k}(b(ceil(feature_info.feature_selection_rate(k)*numel(b)):end)) = 0;
            
            T1 = repmat(L{k},1,1,size(T1,3)) .* T1;
        end
        
        g_f = fft2(T1);
        
        %   update h
        h_f = h_f + (mu * (f_f - g_f));
        
        %   update mu
        mu = min(mu_scale_step * mu, 0.1);
        
        iter = iter+1;
    end
    
    % save the trained filters
    theta_pre_f{k} = f_f;
    %cf_f{k} = f_f;
    
    if seq.frame == 1
        theta_model_f{k} = f_f;
    else
        theta_model_f{k} = feature_info.learning_rate(k)*f_f + (1-feature_info.learning_rate(k))*theta_model_f{k};
    end
end


end

